@@ -1,46 +1,69 @@
 //! The Minesweeper game.
 //!
+//! Check out struct [`Minesweeper`](https://docs.rs/gamie/*/gamie/minesweeper/struct.Minesweeper.html) for more information.
+//!
 //! # Examples
 //!
 //! ```rust
 //! # fn minesweeper() {
-//! use gamie::minesweeper::*;
+//! use gamie::minesweeper::Minesweeper;
+//! use rand::rngs::ThreadRng;
+//!
+//! let mut game = Minesweeper::new(8, 8, 9, &mut ThreadRng::default()).unwrap();
 //!
-//! let mut game = Minesweeper::new(8, 8, 9).unwrap();
-//! game.click(7, 7, true).unwrap();
-//! // ...
 //! game.toggle_flag(3, 2).unwrap();
 //! // ...
+//! game.click(7, 7, true).unwrap();
+//! // ...
 //! # }
 //! ```
 
+#[cfg(feature = "std")]
+use std::collections::VecDeque;
+
+#[cfg(not(feature = "std"))]
+use alloc::{collections::VecDeque, vec::Vec};
+
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+
+use rand::{
+    distributions::{Distribution, Uniform},
+    Rng,
+};
+use snafu::Snafu;
+
 /// The Minesweeper game.
 ///
 /// To avoid unessecary memory allocation, the game board is stored in a single `Vec` rather than a nested one. Use the `get` method to access the board instead of using the `board` field directly.
 ///
-/// If you pass an invalid position to a method, the game will panic. Make sure to check the target position when dealing with user input using `check_position_validity` method.
+/// If you pass an invalid position to a method, the game will panic. Remember to check the target position validity when dealing with user input.
 #[derive(Clone, Debug)]
+#[cfg(feature = "serde")]
+#[derive(Deserialize, Serialize)]
 pub struct Minesweeper {
-    pub board: Vec<MinesweeperCell>,
+    pub board: Vec<Cell>,
     pub height: usize,
     pub width: usize,
-    pub state: MinesweeperState,
+    pub state: GameState,
 }
 
 /// The cell of the Minesweeper board.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
-pub struct MinesweeperCell {
+#[cfg(feature = "serde")]
+#[derive(Deserialize, Serialize)]
+pub struct Cell {
     pub is_mine: bool,
-    pub adjacent_mine: usize,
+    pub mine_adjacent: usize,
     pub is_revealed: bool,
     pub is_flagged: bool,
 }
 
-impl MinesweeperCell {
+impl Cell {
     fn new(is_mine: bool) -> Self {
         Self {
             is_mine,
-            adjacent_mine: 0,
+            mine_adjacent: 0,
             is_revealed: false,
             is_flagged: false,
         }
@@ -49,22 +72,43 @@ impl MinesweeperCell {
 
 /// The Minesweeper game state.
 #[derive(Clone, Debug, Eq, PartialEq)]
-pub enum MinesweeperState {
+#[cfg(feature = "serde")]
+#[derive(Deserialize, Serialize)]
+pub enum GameState {
     Win,
     Exploded(Vec<(usize, usize)>),
     InProgress,
 }
 
 impl Minesweeper {
-    /// Create a new Minesweeper game. Return `Err(MinesweeperError::TooManyMines)` if `height * width < mines`.
-    pub fn new(height: usize, width: usize, mines: usize) -> Result<Self, MinesweeperError> {
+    /// Create a new Minesweeper game.
+    ///
+    /// A mutable reference of a random number generator is required for randomizing mines' position.
+    ///
+    /// Return `Err(MinesweeperError::TooManyMines)` if `height * width < mines`.
+    ///
+    /// # Examples
+    /// ```rust
+    /// # fn minesweeper() {
+    /// use gamie::minesweeper::Minesweeper;
+    /// use rand::rngs::ThreadRng;
+    ///
+    /// let mut game = Minesweeper::new(8, 8, 9, &mut ThreadRng::default()).unwrap();
+    /// # }
+    /// ```
+    pub fn new<R: Rng + ?Sized>(
+        height: usize,
+        width: usize,
+        mines: usize,
+        rng: &mut R,
+    ) -> Result<Self, MinesweeperError> {
         if height * width < mines {
             return Err(MinesweeperError::TooManyMines);
         }
 
-        let board = itertools::repeat_n(MinesweeperCell::new(true), mines)
+        let board = itertools::repeat_n(Cell::new(true), mines)
             .chain(itertools::repeat_n(
-                MinesweeperCell::new(false),
+                Cell::new(false),
                 height * width - mines,
             ))
             .collect();
@@ -73,47 +117,60 @@ impl Minesweeper {
             board,
             height,
             width,
-            state: MinesweeperState::InProgress,
+            state: GameState::InProgress,
         };
-        minesweeper.randomize();
+        minesweeper.randomize(rng).unwrap();
 
         Ok(minesweeper)
     }
 
     /// Randomize the Minesweeper board.
     /// Useful if the first click is on a mine.
-    pub fn randomize(&mut self) {
-        use rand::distributions::{Distribution, Uniform};
+    ///
+    /// A mutable reference of a random number generator is required for randomizing mines' position.
+    pub fn randomize<R: Rng + ?Sized>(&mut self, rng: &mut R) -> Result<(), MinesweeperError> {
+        if self.is_ended() {
+            return Err(MinesweeperError::GameEnded);
+        }
 
-        let mut rng = rand::thread_rng();
         let range = Uniform::from(0..self.height * self.width);
 
         for idx in 0..self.height * self.width {
-            self.board.swap(idx, range.sample(&mut rng));
+            self.board.swap(idx, range.sample(rng));
         }
 
         self.update_around_mine_count();
+
+        Ok(())
     }
 
-    /// Check if the game was already end.
+    /// Check if the game is already ended.
     pub fn is_ended(&self) -> bool {
-        self.state != MinesweeperState::InProgress
+        self.state != GameState::InProgress
     }
 
     /// Get the state of the game.
-    pub fn state(&self) -> &MinesweeperState {
+    pub fn state(&self) -> &GameState {
         &self.state
     }
 
     /// Get a cell reference from the game board.
     /// Panic if the target position is out of bounds.
-    pub fn get(&self, row: usize, col: usize) -> &MinesweeperCell {
+    pub fn get(&self, row: usize, col: usize) -> &Cell {
+        if row >= self.height || col >= self.width {
+            panic!("Invalid position: ({}, {})", row, col);
+        }
+
         &self.board[row * self.width + col]
     }
 
     /// Get a mutable cell reference from the game board.
     /// Panic if the target position is out of bounds.
-    pub fn get_mut(&mut self, row: usize, col: usize) -> &mut MinesweeperCell {
+    pub fn get_mut(&mut self, row: usize, col: usize) -> &mut Cell {
+        if row >= self.height || col >= self.width {
+            panic!("Invalid position: ({}, {})", row, col);
+        }
+
         &mut self.board[row * self.width + col]
     }
 
@@ -131,6 +188,14 @@ impl Minesweeper {
         col: usize,
         auto_flag: bool,
     ) -> Result<bool, MinesweeperError> {
+        if row >= self.height || col >= self.width {
+            panic!("Invalid position: ({}, {})", row, col);
+        }
+
+        if self.is_ended() {
+            return Err(MinesweeperError::GameEnded);
+        }
+
         if !self.board[row * self.width + col].is_revealed {
             self.click_unrevealed(row, col)?;
             Ok(true)
@@ -144,6 +209,14 @@ impl Minesweeper {
     ///
     /// Panic if the target position is out of bounds.
     pub fn toggle_flag(&mut self, row: usize, col: usize) -> Result<(), MinesweeperError> {
+        if row >= self.height || col >= self.width {
+            panic!("Invalid position: ({}, {})", row, col);
+        }
+
+        if self.is_ended() {
+            return Err(MinesweeperError::GameEnded);
+        }
+
         if self.board[row * self.width + col].is_revealed {
             return Err(MinesweeperError::AlreadyRevealed);
         }
@@ -151,20 +224,7 @@ impl Minesweeper {
         self.board[row * self.width + col].is_flagged =
             !self.board[row * self.width + col].is_flagged;
 
-        self.check_game_state();
-
-        Ok(())
-    }
-
-    /// Check if the game was already end and the target position is in the board.
-    pub fn check_position_validity(&self, row: usize, col: usize) -> Result<(), MinesweeperError> {
-        if self.state != MinesweeperState::InProgress {
-            return Err(MinesweeperError::GameEnded);
-        }
-
-        if row >= self.height || col >= self.width {
-            return Err(MinesweeperError::OutOfBounds);
-        }
+        self.check_state();
 
         Ok(())
     }
@@ -175,12 +235,12 @@ impl Minesweeper {
         }
 
         if self.board[row * self.width + col].is_mine {
-            self.state = MinesweeperState::Exploded(vec![(row, col)]);
+            self.state = GameState::Exploded(vec![(row, col)]);
             return Ok(());
         }
 
         self.reveal_from(row * self.width + col);
-        self.check_game_state();
+        self.check_state();
 
         Ok(())
     }
@@ -193,7 +253,7 @@ impl Minesweeper {
     ) -> Result<bool, MinesweeperError> {
         let mut is_changed = false;
 
-        if self.board[row * self.width + col].adjacent_mine > 0 {
+        if self.board[row * self.width + col].mine_adjacent > 0 {
             let mut adjacent_all = 0;
             let mut adjacent_revealed = 0;
             let mut adjacent_flagged = 0;
@@ -213,7 +273,7 @@ impl Minesweeper {
             let adjacent_unrevealed = adjacent_all - adjacent_revealed - adjacent_flagged;
 
             if adjacent_unrevealed > 0 {
-                if adjacent_flagged == self.board[row * self.width + col].adjacent_mine {
+                if adjacent_flagged == self.board[row * self.width + col].mine_adjacent {
                     let mut exploded = None;
 
                     self.get_adjacent_cells(row, col).for_each(|idx| {
@@ -235,14 +295,14 @@ impl Minesweeper {
                     });
 
                     if let Some(exploded) = exploded {
-                        self.state = MinesweeperState::Exploded(exploded);
+                        self.state = GameState::Exploded(exploded);
                         return Ok(true);
                     }
                 }
 
                 if auto_flag
                     && adjacent_unrevealed + adjacent_flagged
-                        == self.board[row * self.width + col].adjacent_mine
+                        == self.board[row * self.width + col].mine_adjacent
                 {
                     self.get_adjacent_cells(row, col).for_each(|idx| {
                         if !self.board[idx].is_flagged && !self.board[idx].is_revealed {
@@ -253,18 +313,16 @@ impl Minesweeper {
                 }
             }
 
-            self.check_game_state();
+            self.check_state();
         }
 
         Ok(is_changed)
     }
 
     fn reveal_from(&mut self, idx: usize) {
-        if self.board[idx].adjacent_mine != 0 {
+        if self.board[idx].mine_adjacent != 0 {
             self.board[idx].is_revealed = true;
         } else {
-            use std::collections::VecDeque;
-
             let mut cell_idxs_to_reveal = VecDeque::new();
             cell_idxs_to_reveal.push_back(idx);
 
@@ -276,7 +334,7 @@ impl Minesweeper {
                 {
                     if !self.board[neighbor_idx].is_flagged && !self.board[neighbor_idx].is_revealed
                     {
-                        if self.board[neighbor_idx].adjacent_mine == 0 {
+                        if self.board[neighbor_idx].mine_adjacent == 0 {
                             cell_idxs_to_reveal.push_back(neighbor_idx);
                         } else {
                             self.board[neighbor_idx].is_revealed = true;
@@ -287,16 +345,16 @@ impl Minesweeper {
         }
     }
 
-    fn check_game_state(&mut self) {
+    fn check_state(&mut self) {
         self.state = if self
             .board
             .iter()
             .filter(|cell| !cell.is_mine)
             .all(|cell| cell.is_revealed)
         {
-            MinesweeperState::Win
+            GameState::Win
         } else {
-            MinesweeperState::InProgress
+            GameState::InProgress
         };
     }
 
@@ -307,24 +365,24 @@ impl Minesweeper {
                 .filter(|idx| self.board[*idx].is_mine)
                 .count();
 
-            self.board[idx].adjacent_mine = count;
+            self.board[idx].mine_adjacent = count;
         }
     }
 
-    fn get_adjacent_cells(&self, row: usize, col: usize) -> MinesweeperAdjacentCells {
-        MinesweeperAdjacentCells::new(row, col, self.height, self.width)
+    fn get_adjacent_cells(&self, row: usize, col: usize) -> AdjacentCells {
+        AdjacentCells::new(row, col, self.height, self.width)
     }
 }
 
 #[derive(Clone)]
-struct MinesweeperAdjacentCells {
-    around: [(i128, i128); 8],
-    board_height: i128,
-    board_width: i128,
+struct AdjacentCells {
+    around: [(isize, isize); 8],
+    board_height: isize,
+    board_width: isize,
     offset: usize,
 }
 
-impl Iterator for MinesweeperAdjacentCells {
+impl Iterator for AdjacentCells {
     type Item = usize;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -342,16 +400,16 @@ impl Iterator for MinesweeperAdjacentCells {
     }
 }
 
-impl MinesweeperAdjacentCells {
+impl AdjacentCells {
     fn new(row: usize, col: usize, board_height: usize, board_width: usize) -> Self {
         let (row, col, board_height, board_width) = (
-            row as i128,
-            col as i128,
-            board_height as i128,
-            board_width as i128,
+            row as isize,
+            col as isize,
+            board_height as isize,
+            board_width as isize,
         );
 
-        MinesweeperAdjacentCells {
+        AdjacentCells {
             around: [
                 (row - 1, col - 1),
                 (row - 1, col),
@@ -369,19 +427,15 @@ impl MinesweeperAdjacentCells {
     }
 }
 
-use thiserror::Error;
-
 /// Errors that can occur.
-#[derive(Debug, Eq, Error, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Snafu)]
 pub enum MinesweeperError {
-    #[error("Too many mines")]
+    #[snafu(display("Too many mines"))]
     TooManyMines,
-    #[error("Position out of bounds")]
-    OutOfBounds,
-    #[error("Clicked an already flagged cell")]
+    #[snafu(display("Clicked an already flagged cell"))]
     AlreadyFlagged,
-    #[error("Clicked an already revealed cell")]
+    #[snafu(display("Clicked an already revealed cell"))]
     AlreadyRevealed,
-    #[error("The game was already ended")]
+    #[snafu(display("The game was already end"))]
     GameEnded,
 }
